import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.netbeans.api.wizard.WizardDisplayer;
import org.netbeans.spi.wizard.DeferredWizardResult;
import org.netbeans.spi.wizard.ResultProgressHandle;
import org.netbeans.spi.wizard.Summary;
import org.netbeans.spi.wizard.Wizard;
import org.netbeans.spi.wizard.WizardController;
import org.netbeans.spi.wizard.WizardException;
import org.netbeans.spi.wizard.WizardPanelProvider;

import org.apache.tools.ant.Project;



public class IhmInstalleur {
	
	private final static double MIN_VERSION = 1.6;
	private final int LARGEUR = 850;
	private final int HAUTEUR = 450;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		String[] nums = System.getProperty("java.version").split("\\.");
		if (Double.parseDouble(nums[0]+"."+nums[1]) < MIN_VERSION) {
			System.out.println("Cette application demande une version de Java supérieure ou égale à " + MIN_VERSION);
			System.exit(0); 
		}
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new FenetreLangues();
			}
		});
	}
	
	public IhmInstalleur(int indexLangue, Properties prop) {
		
		Toolkit tk = Toolkit.getDefaultToolkit();
		Dimension ecran = tk.getScreenSize();
		int y = (int) ((ecran.getWidth() - LARGEUR) / 2 );
		int x = (int) ((ecran.getHeight() - HAUTEUR) / 2 );
		
		String[] langues = (prop.getProperty("langues.abbreviations")).split(" ");
		String langue = langues[indexLangue];
		String[] compLangue = langue.split("_");
		ResourceBundle messages = ResourceBundle.getBundle("locale.MessageBundle", new Locale(compLangue[0], compLangue[1]), this.getClass().getClassLoader());
		FournisseurPages fournisseur = new FournisseurPages(messages);
		Wizard assistant = fournisseur.createWizard();
		Map<String,Object> initial = new Hashtable<String,Object>();
		initial.put("messages", messages);
		initial.put("prop", prop);
		WizardDisplayer.showWizard(assistant, new Rectangle (x, y, LARGEUR, HAUTEUR), null, initial);
	}

}

class FenetreLangues extends JFrame {
	
	private final String INSTALLEUR_PROPERTIES_FILE  = "installeur.properties";
	private final int LARGEUR = 270;
	private final int HAUTEUR = 150;
	private JComboBox listeLangues;
	
	public FenetreLangues() {
		JPanel panel = new JPanel(new BorderLayout(20, 20));
		JPanel panelLangue = new JPanel(new BorderLayout(10, 10));
		JPanel panelBoutons = new JPanel();
		panelLangue.add(new JLabel("Please select a language below"), BorderLayout.NORTH);
		
		final Properties prop = new Properties();
		try {
			prop.load(ClassLoader.getSystemResourceAsStream(INSTALLEUR_PROPERTIES_FILE));
		} catch (Exception e) {
			System.out.println("Error: I cannot find the properties file\n");
			System.out.println();
			e.printStackTrace();
			System.exit(1);
		}
		
		String[] langueComplet = prop.getProperty("langues.completes").split(" ");
		listeLangues = new JComboBox(langueComplet);
		listeLangues.setSelectedIndex(1);
		
		
		JButton boutonDemarrage = new JButton("Launch installer");
		boutonDemarrage.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
				new IhmInstalleur(listeLangues.getSelectedIndex(), prop);
			}
		});
		JButton boutonQuitter = new JButton("Quit");
		boutonQuitter.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		panelLangue.add(listeLangues, BorderLayout.CENTER);
		panelBoutons.add(boutonDemarrage);
		panelBoutons.add(boutonQuitter);
		panel.add(panelBoutons, BorderLayout.SOUTH);
		panel.add(panelLangue, BorderLayout.NORTH);
		getContentPane().add(panel);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setSize(LARGEUR, HAUTEUR);
		setLocationRelativeTo(null);
		setVisible(true);
	}
	
	/*public void lancerInstalleur() {
		dispose();
		new IhmInstalleur((String)listeLangues.getSelectedItem());
	}*/
}

class FournisseurPages extends WizardPanelProvider {
	
	private ResourceBundle messages;
	
	public FournisseurPages(ResourceBundle messages) {
		super(messages.getString("gui.titre"),
			  new String[] {"bienvenue", "informations", "licence", "mysql_parametres", "servlet_parametres", "osames_parametres", "resume"},
			  new String[] {messages.getString("gui.bienvenue.description"), messages.getString("gui.informations.description"),
					  		messages.getString("gui.licence.description"), messages.getString("gui.mysqlparams.description"),
					  		messages.getString("gui.servletparams.description"), messages.getString("gui.osamesparams.description"),
					  		messages.getString("gui.resume.description")});
		messages = messages;
	}
	
	protected JComponent createPanel(WizardController wizardController, String id, Map settings) {
		
		if ("bienvenue".equals(id))
			return new Bienvenue(settings);
		else if ("informations".equals(id))
			return new Information(settings);
		else if ("licence".equals(id))
			return new Licence(settings);
		else if ("mysql_parametres".equals(id))
			return new MysqlParametres(settings);
		else if ("servlet_parametres".equals(id))
			return new ServletParametres(settings);
		else if ("osames_parametres".equals(id))
			return new OsamesParametres(settings);
		else if ("resume".equals(id))
			return new Resume(settings);
		return null;
	}

	public boolean cancel(Map settings) {
		Object[] options = {"D'accord", "Annuler l'action"};
		if (JOptionPane.showOptionDialog(null, ((ResourceBundle)settings.get("messages")).getString("gui.quitter.message"),
									 "Quitter l'installation ?",
									 JOptionPane.OK_CANCEL_OPTION,
									 JOptionPane.QUESTION_MESSAGE,
									 null,
									 options, options[1]) == JOptionPane.OK_OPTION) {
			System.exit(0);
			return true;
		}
		return false;
	}
	
	public Object finish (Map wizardData) throws WizardException {
		
	    return new Result();
	}
	
}

class Result extends DeferredWizardResult {
	
	public Result () {
		// Si on renvoie true, alors on peut avorter l'installation
		// au moyen de la fermeture de l'installeur
		super (true);
	}

	public void start (Map settings, ResultProgressHandle progress) {
		
		try {
			Installeur installeur = new Installeur(settings);
			Project projet = installeur.creerProjet();
			
			progress.setProgress((String)settings.get("gui.wizardresult.creer.servlet"), 0, 4);
	    	installeur.creerServlet(projet);
	
		    progress.setProgress((String)settings.get("gui.wizardresult.creer.conf"), 1, 4);
	    	installeur.creerConf(projet);
	
	        progress.setProgress((String)settings.get("gui.wizardresult.creer.base"), 2, 4);
	    	installeur.creerBase(projet);
	
	        progress.setProgress((String)settings.get("gui.wizardresult.charger.communes"), 3, 4);
	    	installeur.chargerCommunes(projet);
	        
		    // Le deuxième paramètre de finished() permet de placer un objet qui sera retourné
	    	// par showWizard()
		    progress.finished(Summary.create(new String[] {((ResourceBundle)settings.get("messages")).getString("gui.wizardresult.termine")}, null));
		} catch (Exception e) {
			System.out.println(((ResourceBundle)settings.get("messages")).getString("erreur.installation") + "\n");
			progress.failed(((ResourceBundle)settings.get("messages")).getString("erreur.installation"), true);
			e.printStackTrace();
		}
	}

}


